#!/bin/sh
RAD_RET=255
RAD_F_LOCK="/tmp/synoradius.lock"
RAD_F_PID="/var/packages/RadiusServer/target/var/run/radiusd/radiusd.pid"
RAD_F_LISTEN="/usr/local/synoradius/rad_listen"
RAD_F_TLS_MIN="/usr/local/synoradius/rad_tls_min"
RAD_F_TLS_MAX="/usr/local/synoradius/rad_tls_max"

SYNO_CONF_JSON="/usr/local/synoradius/synoconf.json"

PATH_IFCFG="/sbin/ifconfig"
PATH_AWK="/usr/bin/awk"
PATH_CUT="/usr/bin/cut"
PATH_SORT="/usr/bin/sort"
PATH_UNIQ="/usr/bin/uniq"

UseLocal() {
	local BACK_END_TYPE=`/usr/syno/bin/synogetkeyvalue  /usr/local/synoradius/synoconf back_end_type`

	if [ "xlocal" != "x${BACK_END_TYPE}" ]; then
		echo "\$INCLUDE /usr/local/synoradius/rad_site_def_local" > "/usr/local/synoradius/rad_site_def"
		echo "\$INCLUDE /usr/local/synoradius/rad_site_inn_local" > "/usr/local/synoradius/rad_site_inn"
		/usr/syno/bin/synosetkeyvalue /usr/local/synoradius/synoconf back_end_type "local"
	fi
	return 0
}

SetTlsCfg() {
	TLS_MIN="1.2"
	TLS_MAX="1.2"
	if [ -f $SYNO_CONF_JSON ]; then 
		TLS_MIN=`/bin/cat $SYNO_CONF_JSON | /bin/jq -r .tls_min_version`
		if [ "$TLS_MIN" != "1.0" -a "$TLS_MIN" != "1.2" ]; then
			TLS_MIN="1.2"
		fi
		TLS_MAX=`/bin/cat $SYNO_CONF_JSON | /bin/jq -r .tls_max_version`
		if [ "$TLS_MAX" != "1.0" -a "$TLS_MAX" != "1.2" ]; then
			TLS_MAX="1.2"
		fi
	fi
	/usr/syno/bin/synosetkeyvalue $RAD_F_TLS_MIN tls_min_version $TLS_MIN
	/usr/syno/bin/synosetkeyvalue $RAD_F_TLS_MAX tls_max_version $TLS_MAX

	setCipherList
}

setCipherList() {
	DEFAULT_CIPHER_LIST="ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:DHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256:ECDHE-ECDSA-AES128-SHA:ECDHE-RSA-AES256-SHA384:ECDHE-RSA-AES128-SHA:ECDHE-ECDSA-AES256-SHA384:ECDHE-ECDSA-AES256-SHA:ECDHE-RSA-AES256-SHA:DHE-RSA-AES128-SHA256:DHE-RSA-AES128-SHA:DHE-RSA-AES256-SHA256:DHE-RSA-AES256-SHA:ECDHE-ECDSA-DES-CBC3-SHA:ECDHE-RSA-DES-CBC3-SHA:EDH-RSA-DES-CBC3-SHA:AES128-GCM-SHA256:AES256-GCM-SHA384:AES128-SHA256:AES256-SHA256:AES128-SHA:AES256-SHA:DES-CBC3-SHA:!DSS"
	PATH_CIPHER_LIST="/var/packages/RadiusServer/target/etc/cipher_list"

	cipher_list=`/bin/cat /usr/syno/etc/security-profile/tls-profile/config/radiusserver.conf`
	TLS_MIN_VER=`/usr/syno/bin/synogetkeyvalue $RAD_F_TLS_MIN tls_min_version`
	if [ "1.0" = "$TLS_MIN_VER" ]; then
		/usr/syno/bin/synosetkeyvalue $PATH_CIPHER_LIST "cipher_list" "DEFAULT@SECLEVEL=1"
	elif [ "$cipher_list" = "" ]; then
		/usr/syno/bin/synosetkeyvalue $PATH_CIPHER_LIST "cipher_list" $DEFAULT_CIPHER_LIST
	else
		echo $cipher_list > $PATH_CIPHER_LIST
fi
}

SetListen() {
	echo "" > $RAD_F_LISTEN
	echo -e "listen {" >> $RAD_F_LISTEN
	echo -e "\ttype = auth" >> $RAD_F_LISTEN
	echo -e "\tipaddr = *" >> $RAD_F_LISTEN
	echo -e "\t\$INCLUDE /usr/local/synoradius/rad_port_auth" >> $RAD_F_LISTEN
	echo -e "}" >> $RAD_F_LISTEN
	return 0
}

StartRAD() {
	for i in `seq 1 1 3`; do
		sleep 3
		/var/packages/RadiusServer/target/sbin/radiusd
		if [ 0 -eq $? ]; then
			return 0
		fi
	done
	return 255
}

StopRAD() {
	if [ -f $RAD_F_PID ]; then
		kill -TERM `cat ${RAD_F_PID}`
	fi
}

ReloadRAD() {
	if [ -f $RAD_F_PID ]; then
		kill -HUP `cat ${RAD_F_PID}`
	fi
}

{
flock -x 0

case $1 in
	start)
		SetTlsCfg
		SetListen
		StartRAD
		RAD_RET=$?
	;;
	uselocal)
		UseLocal
		StopRAD
		StartRAD
		RAD_RET=$?
	;;
	restart)
		StopRAD
		SetTlsCfg
		SetListen
		StartRAD
		RAD_RET=$?
	;;
	reloadCipherList)
		StopRAD
		setCipherList
		StartRAD
		RAD_RET=$?
	;;
	reload)
		ReloadRAD
		RAD_RET=0
	;;
esac

flock -u 0
} 0<>$RAD_F_LOCK
exit $RAD_RET

